@@ -31,8 +31,7 @@ import asyncio
 import base64
 import json
 import logging
-from logging import Logger, config
-from os.path import exists
+from logging import Logger
 from typing import Awaitable, Callable, List, Union, Dict
 
 import aiohttp
@@ -64,7 +63,8 @@ class _RestClient:
         await self._con.close()
 
     async def connect(self):
-        self._con = aiohttp.ClientSession(base_url=self._url)
+        # set trust_env to True to enable env variable settings for network proxy support
+        self._con = aiohttp.ClientSession(base_url=self._url, trust_env=True)
 
     async def close(self):
         await self._con.close()
@@ -136,6 +136,7 @@ class VantiqResources:
     IMAGES = 'system.images'
     K8S_CLUSTERS = 'system.k8sclusters'
     K8S_INSTALLATIONS = 'system.k8sinstallations'
+    LLMS = 'system.llms'
     NAMESPACES = 'system.namespaces'
     ORGANIZATIONS = 'system.organizations'
     PROFILES = 'system.profiles'
@@ -143,6 +144,7 @@ class VantiqResources:
     RULES = 'system.rules'
     SCHEDULED_EVENTS = 'system.scheduledevents'
     SECRETS = 'system.secrets'
+    SEMANTIC_INDEXES = 'system.semanticindexes'
     SERVICES = 'system.services'
     SITUATIONS = 'system.situations'
     SOURCES = 'system.sources'
@@ -350,8 +352,8 @@ class Vantiq:
     This class consists of operations that can be performed using the Vantiq system. More information about
     these operations can be found in the Vantiq Resource Reference Guide and the Vantiq API Reference Guide.
 
-    Logging for this class can be configured using a `logging.ini` file that is present in the working directory
-    of the running program.
+    Logging for this class is done through a standard Python logger named "Vantiq".  It is assumed that the
+    client of this library will configure the logger as desired.
 
     The Vantiq object instance can be used as a context manager.
         async with Vantiq(<server url>) as client:
@@ -393,10 +395,6 @@ class Vantiq:
 
 
         """
-        if exists('logger.ini'):
-            logging.config.fileConfig('logger.ini', disable_existing_loggers=False)
-        else:
-            print('No logger.ini file found.')
         self._vlog: Logger = logging.getLogger(self.__class__.__name__)
 
         if server.endswith('/'):
@@ -807,7 +805,8 @@ class Vantiq:
             resource : str
                 Name of the Vantiq resource into which to insert.
             instance : dict
-                The values to be inserted.
+                The values to be inserted.  The key names in the _instance_ parameter
+                must match the property names in the Vantiq object.
 
         Returns:
             VantiqResponse indicating success/failure of the operation and the value inserted.
@@ -837,7 +836,8 @@ class Vantiq:
                 resource : str
                     Name of the Vantiq resource into which to upsert.
                 instance : dict
-                    The values to be upserted.
+                    The values to be upserted.   The key names in the _instance_ parameter
+                    must match the property names in the Vantiq object.
 
             Returns:
                 VantiqResponse indicating success/failure of the operation and the value upserted.
@@ -871,7 +871,8 @@ class Vantiq:
                 resource_id : str
                     The key of the record to look up for replacement.  The _id property can be used/
                 instance : dict
-                    The values to be updated.
+                    The values to be updated. The key names in the _instance_ parameter
+                    must match the property names being updated in the Vantiq object.
 
             Returns:
                 VantiqResponse indicating success/failure of the operation and the value updated.
@@ -962,10 +963,10 @@ class Vantiq:
 
         Examples:
         ::
-            vr: VantiqResponse = await client.upload(VantiqResources.DOCUMENTS, 'iamge/png', '/file/name.png')
+            vr: VantiqResponse = await client.upload(VantiqResources.DOCUMENTS, 'iamge/png', 'path/name.png')
             if vr.is_success:
-                # Here, we will have uploaded a file named `/file/name.png`,
-                # and created a Document named `/file/name.png`.
+                # Here, we will have uploaded a file named `path/name.png`,
+                # and created a Document named `path/name.png`.
 
             vr: VantiqResponse = await client.upload(VantiqResource.DOCUMENTS, 'text/plain', None,
                                                      'my document', 'some content for my document')
@@ -1018,7 +1019,7 @@ class Vantiq:
                                   'Unexpected error during {0} operation.',
                                   ['upload']) from e
 
-    async def count(self, resource: str, where: Union[dict, None]) -> VantiqResponse:
+    async def count(self, resource: str, where: Union[dict, None] = None) -> VantiqResponse:
         """(Async) Return the number of items in a Vantiq resource that satisfy the where clause
 
         Parameters:
@@ -1044,6 +1045,7 @@ class Vantiq:
             method = 'GET'
             path = self._build_path(resource, None)
             resp = await self._perform_operation(operation, method, path, query_params, False)
+            resp.body = {}  # After a count, we don't need to return the data fetched (since we decided it anyway)
             return resp
         except VantiqException:
             # If we've already handled or wrapped it, just pass it along
@@ -1220,10 +1222,10 @@ class Vantiq:
             return None
 
     async def subscribe(self, resource: str, resource_id: str, operation: Union[str, None],
-                        callback: Callable[[str, dict], Awaitable[None]], params: dict) -> VantiqResponse:
+                        callback: Callable[[str, dict], Awaitable[None]], params: dict = None) -> VantiqResponse:
         """(Async) Subscribe to an event from the Vantiq server.
 
-        Subscribes to a specific topic, source, or type event.
+        Subscribes to a specific topic, source, service, or type event.
 
         For sources, this will subscribe to message arrival events.  The name of the
         source is required (e.g. "MySource").
@@ -1231,6 +1233,9 @@ class Vantiq:
         For topics, this will subscribe to any messages published on that topic.  The
         name of the topic is required (e.g. "/some/topic").
 
+        For services, this will subscribe to any messages published to the named service event. The name of the
+        service and service event is required, and should be passed into the resource id as service_name/event_name.
+
         For types, this will subscribe to the specified type event.  The name of the
         type and the operation (i.e. "insert", "update", or "delete") are required.
 
@@ -1253,7 +1258,7 @@ class Vantiq:
                     value : dict -- the value of the event (type inserted, topic contents, etc.)
 
             params : dict
-                Parameters for the subscription
+                (optional) Parameters for the subscription. May be subscription dependent, but can usually be ignored.
         Returns:
             VantiqResponse indicating the success of the operation.
 
@@ -1264,7 +1269,7 @@ class Vantiq:
             path = "/" + resource[len(_SYSTEM_PREFIX):] + resource_id  # .removeprefix(_SYSTEM_PREFIX) + resource_id
         else:
             path = "/" + resource[len(_SYSTEM_PREFIX):] + '/' + resource_id  # .removeprefix(_SYSTEM_PREFIX) ...
-        if resource in [VantiqResources.SOURCES, VantiqResources.TOPICS]:
+        if resource in [VantiqResources.SERVICES, VantiqResources.SOURCES, VantiqResources.TOPICS]:
             if operation:
                 raise VantiqException('io.vantiq.python.operationillegal',
                                       "Operation only support for {0}",
@@ -1278,7 +1283,7 @@ class Vantiq:
             path += "/" + operation.lower()
         else:
             raise VantiqException('io.vantiq.python.invalidsubscribetype',
-                                  "Only 'topics', 'sources' and 'types' support subscribe",
+                                  "Only 'topics', 'sources', 'services', and 'types' support subscribe",
                                   [])
 
         if self._subscriber is None:
@@ -1325,7 +1330,7 @@ class _VantiqSubscriber:
         self.parent = parent
         self.connected = False
         self.connected_future: asyncio.Future = asyncio.get_running_loop().create_future()
-        self.connection: websockets.ClientConnection = None
+        self.connection: websockets.ClientProtocol = None
         self.url = None
         self._vlog = logging.getLogger(self.__class__.__name__)
         self.subscriptions: Dict[str, bool] = {}
